# README #

## Summary ##
EqDsatur is an algorithm to color a graph equitably, that is, the sizes of the color classes differ from each other by at most one. For this matter we use a branch and bound algorithm coupled with several pruning methods. For further information we refer to our paper [1].

## Technical ##
The tool was developed with C++11 on Linux. Furthermore, we made usage of the BOOST Graph Library and have used the GNU/GCC compiler (4.8.4) with several flags, *-O3* among others, but feel free to use any other C++ compiler which supports the C++11 standard. In order to compile our program you can use our provided makefile.

## Usage ##

```
./eqDsatur [N | R] [N | C] nPruningRule timeLimit threshold [FILE | n p SEED] ClPerc NoColorExclColor NoColorCard NoColorExclColorClq doMaxFlow nPruningComb nCliqueDecomp (starting_LB starting_UB)

```

### Parameters ###
1. *[N | R]*: You can choose between N for normal mode, to select a specific graph as
   input, or R for random graph mode, to generate a random graph (Erdos-Renyi) as input.

2. *[N | C]*: You can choose between N for normal mode, to select the normal dSatur
   algorithm, or C for clique mode, which uses our algorithm.

3. *nPruningRule*: Only works in clique mode. Our pruning rule (based on max.-flow problem) [1] will be asked for at every *nPruningRule*-th node in the enumeration tree.

4. *timeLimit*: Maximum timelimit in seconds.

5. *threshold*: Threshold used for the vertex selection strategy. Currently passVSS [2] is implemented and according to [2] a threshold value of 3 is recommended.

6. * *FILE*: In normal mode: path to graph's resource file.
   * *n p SEED*: In random graph mode: we use various parameters for the generation of a random graph. The first one states the number of vertices in the random graph. The second one states the density/probability which is used for generating edges between each pair of nodes. Self-loops are not considered. The third one is an initialization seed. We can force the internal random graph generator to generate a specific random graph which is identified by this value. This is useful for comparing results with the same random graph. Is the seed value set to zero, then we get a new random graph everytime, that is, we do not get the same random graph.

7. *ClPerc*: Percentage value between 0 and 1. If our clique is colored by this value then we are looking for a better one, if one exists, otherwise we keep the old cliques. The default value is 0.

8. *NoColorExclColor*: Set to 1 in order to enable the use of combinatorial pruning rule (1) [1]. Set 0 otherwise.

9. *NoColorCard*: Set to 1 in order to enable the use of combinatorial pruning rule (2) [1]. Set 0 otherwise.

10. *NoColorExclColorClq*: Set to 1 in order to enable the use of combinatorial pruning rule (3) [1]. Set 0 otherwise.

11. *doMaxFlow*: Set to 1 in order to use the maximum flow algorithm for pruning. Set 0 otherwise.

12. *nPruningComb*: like 3. - only for combinatorial pruning rules.

13. *nCliqueDecomp*: like 3. - only for independent clique decomposition.

14. *starting_LB*: use specific starting lower bound instead of computing a maximum clique. **currently disabled**

15. *starting_UB*: use specific starting upper bound instead of computing an equitable coloring. **currently disabled**

## References ##
[1] A. M.C.A. Koster, R. B. Scheidweiler, M. Tieves: *A flow based pruning scheme for enumerative equitable coloring algorithms*

[2] I. Mendez Diaz, G. Nasini, D. Severin: *A DSATUR-based Algorithm for the Equitable Coloring Problem*, Computer & Operations Research 57 (2015), 41-50
