// Authors: Sven Förster, Duc Thanh Tran

#ifndef BOOST_HPP
#define BOOST_HPP

#include <boost/graph/adjacency_list.hpp>
#include <boost/graph/graph_traits.hpp>

#include <vector>

namespace EqDsatur {

/// references original vertex in our graph
struct ref_vertex_t {
	typedef boost::vertex_property_tag kind;
};

/// denotes an undirected graph. The hashset is used to store outer edges, the vector structure
/// in the second parameter denotes the storage container for nodes. We choose the hashset in order
/// to avoid parallel edges.
using Graph = boost::adjacency_list<boost::hash_setS, boost::vecS, boost::undirectedS>;

using TraitsGraphFlow = boost::adjacency_list_traits<boost::slistS, boost::vecS, boost::directedS>;
// NOTE: 1. param: slistS gives weird boost compiler error since upgrading to (X)Ubuntu 16.04
using GraphFlow = boost::adjacency_list<boost::vecS, boost::vecS, boost::directedS, boost::property<ref_vertex_t, int>,
															 boost::property<boost::edge_capacity_t, long,
															 boost::property<boost::edge_residual_capacity_t, long,
															 boost::property<boost::edge_reverse_t, TraitsGraphFlow::edge_descriptor> > > >;

using Vertex = boost::graph_traits<Graph>::vertex_descriptor;
using Edge = boost::graph_traits<Graph>::edge_descriptor;
using VertexIt = boost::graph_traits<Graph>::vertex_iterator;
using EdgeIt = boost::graph_traits<Graph>::edge_iterator;
using OutEdgeIt = boost::graph_traits<Graph>::out_edge_iterator;
using AdjacentIt = boost::graph_traits<Graph>::adjacency_iterator;
using InvAdjacentIt = Graph::inv_adjacency_iterator;

using VertexFord = boost::graph_traits<GraphFlow>::vertex_descriptor;
using EdgeFord = boost::graph_traits<GraphFlow>::edge_descriptor;
using VertexFordIt = boost::graph_traits<GraphFlow>::vertex_iterator;
using EdgesOutFordIt = boost::graph_traits<GraphFlow>::out_edge_iterator;
using EdgeFordIter = boost::graph_traits<GraphFlow>::edge_iterator;

using RefVertexMap = boost::property_map<GraphFlow, ref_vertex_t>::type;
using CapacityMap = boost::property_map<GraphFlow, boost::edge_capacity_t>::type;
using ReverseEdgeMap = boost::property_map<GraphFlow, boost::edge_reverse_t>::type;
using ResidualCapacityMap = boost::property_map<GraphFlow, boost::edge_residual_capacity_t>::type;

} // namespace EqDsatur

#endif
