// Authors: Sven Förster, Duc Thanh Tran

#include "input.hpp"
#include "boost.hpp"

#include <iostream>
#include <fstream>

namespace EqDsatur {

Graph Input::readGraph(const std::string &filename) noexcept {
	const auto index = filename.find_last_of(".");
	const auto fileFormat = (index != std::string::npos) ? filename.substr(index+1) : "";

	return [filename, fileFormat]() {
		if (fileFormat == "col") {
			return readGraphCol(filename);
		}
		else {
			Graph emptyGraph;
			return emptyGraph;
		}
	}();
}

Graph Input::readGraphCol(const std::string &filename) noexcept {
	std::ifstream file(filename, std::ios::in);
	Graph g;

	if (!file.good()) {
		std::cout << "Cannot open .col file: " << filename << "\n";
		return g;
	}

	char controlChar = 0;
	std::string tmp;
	int nVertices, nEdges;
	Vertex sourceVertex, targetVertex;

	Edge edge;
	while (file >> controlChar) {
		switch (controlChar) {
			case 'c':
				// comment line: skip it
				getline(file, tmp);
				break;

			case 'p':
				// parameter line: denotes total number of vertices and edges
				file >> tmp;
				if (tmp.compare("edge") != 0) {
					std::cout << "Keyword \"edge\" not found!\n";
					return false;
				}

				file >> nVertices;
				file >> nEdges;
				for (auto i = 0; i < nVertices; ++i) { boost::add_vertex(g); }
				break;

			case 'e':
				// edge line
				file >> sourceVertex;
				file >> targetVertex;

				// node indices start at 1 in DIMACS file format
				// nodes in boost graphs start with zero
				boost::add_edge(sourceVertex-1, targetVertex-1, g);
				break;

			default:
				// unknown control character
				std::cout << "Unknown control character \"" << controlChar << "\" in file " << filename << "\n";
				return g;
		}
	}
	return g;
}

} // namespace EqDsatur
