// Author: Duc Thanh Tran

#ifndef TIMER_HPP
#define TIMER_HPP

#include <ctime>

namespace EqDsatur {
/// Times procedures in our algorithm. Besides we can specify a certain double-variable
/// and when Timer-instance is destroyed, the variable is updated by its life-time.
class Timer {
public:
    Timer() = default;
    explicit Timer(double &timeToSaveTo) : timeVariable(&timeToSaveTo) {}
    ~Timer() {
        if (timeVariable) {
            *timeVariable += static_cast<double>(std::clock() - startTime_) / static_cast<double>(CLOCKS_PER_SEC);
        }
    }

    inline double elapsed() const noexcept {
        return static_cast<double>(std::clock() - startTime_) / static_cast<double>(CLOCKS_PER_SEC);
    }

private:
    std::clock_t startTime_ = std::clock();
    double *timeVariable = nullptr;
};

} // namespace EqDsatur

#endif
